package sample.activity;

import com.poscoict.glueframework.biz.activity.GlueActivity;
import com.poscoict.glueframework.biz.activity.GlueActivityConstants;
import com.poscoict.glueframework.biz.control.GlueBizControlConstants;
import com.poscoict.glueframework.context.GlueContext;
import com.poscoict.glueframework.dao.jdbc.GlueGenericJdbcDao;
import com.poscoict.glueframework.dao.manager.GlueNamedParameterUtils;
import com.poscoict.glueframework.dao.manager.GlueQueryDefinition;
import com.poscoict.glueframework.dao.vo.GlueParameter;

import java.util.*;

/**
 * <pre>
 * <b>activity property</b>
 *
 * - dao : dao bean id
 * - sql-key : query id
 * - dbms-type : db 유형
 * - page-size : 한 페이지에 출력한 row 수
 * - cur-page : 현재 페이지 번호
 * - result-key : 특정 페이지용 data list를 나타내는 key.
 *                전체 data 수는 result-key에 접미사 "_total" 을 key로 함.
 * - param-count : ? 를 사용하는 SQL 용
 * - param# : ? 를 사용하는 SQL 용
 * - param-bindings : named(:name) parameter를 사용하는 SQL용
 *
 * </pre>
 */
public class PageSet extends GlueActivity<GlueContext> {
    // Do not define the Class Member Variable!!
    public String runActivity(GlueContext ctx) {
        String daoKey = this.getProperty(GlueActivityConstants.DAO); // "dao"
        String sqlKey = this.getProperty(GlueActivityConstants.SQLKEY); // "sql-key"
        String dbmsType = this.getProperty("dbms-type");
        String pageSize = this.getProperty("page-size");
        String curPageKey = this.getProperty("cur-page");
        String resultKey = this.getProperty(GlueActivityConstants.RESULT_KEY); // "result-key"
        String paramCount = this.getProperty(GlueActivityConstants.PARAM_COUNT); // "param-count"
        String paramBindings = this.getProperty(GlueActivityConstants.PARAM_BINDINGS);// "param-bindings"

        GlueGenericJdbcDao dao = (GlueGenericJdbcDao) this.getDao(daoKey);
        GlueQueryDefinition def = dao.getQueryManager().getQueryDefinition(sqlKey);
        boolean isNamed = (def != null) && def.isNamedQuery();

        String sql = def.getQueryStatement();
        StringBuilder countSql = new StringBuilder();
        StringBuilder pageSql = new StringBuilder();
        if ("oracle".equalsIgnoreCase(dbmsType)) {
            countSql.append("select count(*) totalRowCnt from (").append(sql).append(")");
            pageSql.append("select * from ( select rownum as no, t.* from (").append(sql).append(") t ) ");
            if (isNamed) {
                pageSql.append(" where no > (:pagesize * (:curpage-1)) ");
                pageSql.append("   and no <= (:pagesize * :curpage)");
            } else {
                pageSql.append(" where no > (? * (?-1)) ");
                pageSql.append("   and no <= (? * ?)");
            }
        } else {
            // DBMS에 따라 count()와 rownum  사용 sql 을 생성함.
        }

        List<Map<?, ?>> pageSet = null;
        List<Map<?, ?>> count = null;
        if (isNamed) {
            Map<String, String> nameMap = this.parseProperty(dao, sqlKey, paramBindings);
            Map<String, Object> paramMap = makeMapParameter(nameMap, 0, ctx);
            count = dao.findByQueryStatement(countSql.toString(), new GlueParameter<Map<String, Object>>(paramMap));

            paramMap.put("pagesize", pageSize);
            paramMap.put("curpage", getValueFromContext(ctx, curPageKey));
            pageSet = dao.findByQueryStatement(pageSql.toString(), new GlueParameter<Map<String, Object>>(paramMap));
        } else {
            int paramCnt = (paramCount == null) ? 0 : Integer.parseInt(paramCount);
            List<Object> listParam = makeListParameter(paramCnt, 0, ctx);
            count = dao.findByQueryStatement(countSql.toString(), new GlueParameter<List<Object>>(listParam));

            listParam.add(pageSize);
            listParam.add(getValueFromContext(ctx, curPageKey));
            listParam.add(pageSize);
            listParam.add(getValueFromContext(ctx, curPageKey));
            pageSet = dao.findByQueryStatement(pageSql.toString(), new GlueParameter<List<Object>>(listParam));
        }

        ctx.put(resultKey + "_total", count.get(0).get("totalRowCnt"));
        ctx.put(resultKey, pageSet);

        return GlueBizControlConstants.SUCCESS;
    }

    private Object getValueFromContext(GlueContext ctx, String key) {
        Object value = ctx.get(key);
        if (value instanceof Object[]) {
            return ((Object[]) value)[0];
        } else {
            return value;
        }
    }

    /**
     * @param dao
     * @param queryKey
     * @param str
     * @return
     */
    private Map<String, String> parseProperty(GlueGenericJdbcDao dao, String queryKey, String str) {
        Set<String> nameSet = GlueNamedParameterUtils.parseSqlStatement(dao.getQueryManager().getQueryDefinition(queryKey).getQueryStatement());
        if ((nameSet == null) || (nameSet.size() < 1)) {
            return null;
        }

        Map<String, String> keymap = new HashMap<String, String>();
        for (String named : nameSet) {
            keymap.put(named, named);
        }

        if ((str == null) || (str.trim().length() < 1)) {
            return keymap;
        }

        StringTokenizer tokenizer = new StringTokenizer(str, GlueActivityConstants.PARAM_SEPERATORS);
        while (tokenizer.hasMoreElements()) {
            String token = tokenizer.nextToken();
            String pair[] = token.split(GlueActivityConstants.EQUALS, 2);
            if (nameSet.contains(pair[0])) {
                pair[1] = pair[1] == null ? pair[0] : pair[1];
                keymap.put(pair[0], pair[1]);
            }
        }
        return keymap;
    }

    /**
     * @param nameMap
     * @param idx
     * @param data
     * @return
     */
    private Map<String, Object> makeMapParameter(Map<String, String> nameMap, int idx, Map<?, ?> data) {
        if ((nameMap == null) || (nameMap.size() < 1)) {
            return null;
        }
        Map<String, Object> args = new HashMap<String, Object>();
        for (String keyInSql : nameMap.keySet()) {
            String keyInData = nameMap.get(keyInSql);
            Object value = data.get(keyInData);
            if (value instanceof Object[]) {
                args.put(keyInSql, ((Object[]) value)[idx]);
            } else {
                args.put(keyInSql, value);
            }
        }
        return args;
    }

    /**
     * @param paramCnt
     * @param idx
     * @param ctx
     * @return
     */
    private List<Object> makeListParameter(int paramCnt, int idx, Map<?, ?> ctx) {
        if (paramCnt == 0) {
            return null;
        }
        List<Object> args = new ArrayList<Object>();
        for (int i = 0; i < paramCnt; i++) {
            Object value = ctx.get(this.getProperty(GlueActivityConstants.PARAM + i));
            if (value == null) {
                args.add(null);
            } else if (value instanceof Object[]) {
                args.add(((Object[]) value)[idx]);
            } else {
                args.add(value);
            }
        }
        return args;
    }
}